@@ -2,6 +2,7 @@ from itertools import count
 from . import boundattributes
 from . import classregistry
 from . import events
+from . import sresults
 from . import styles
 from . import sqlbuilder
 from .styles import capword
@@ -291,46 +292,79 @@ class RelatedJoin(MultipleJoin):
 
 
 class OtherTableToJoin(sqlbuilder.SQLExpression):
-    def __init__(self, otherTable, otherIdName, interTable, joinColumn):
+    def __init__(self, otherTable, otherIdName, interTable, joinColumn, alias):
         self.otherTable = otherTable
         self.otherIdName = otherIdName
         self.interTable = interTable
         self.joinColumn = joinColumn
+        self.alias = alias
 
     def tablesUsedImmediate(self):
-        return [self.otherTable, self.interTable]
+        return [
+            '%s  %s' % (self.otherTable, self.alias)
+            if self.alias else self.otherTable,
+            self.interTable,
+        ]
 
     def __sqlrepr__(self, db):
-        return '%s.%s = %s.%s' % (self.otherTable, self.otherIdName,
-                                  self.interTable, self.joinColumn)
+        return '%s.%s = %s.%s' % (
+            self.alias if self.alias else self.otherTable,
+            self.otherIdName, self.interTable, self.joinColumn)
 
 
 class JoinToTable(sqlbuilder.SQLExpression):
-    def __init__(self, table, idName, interTable, joinColumn):
+    def __init__(self, table, idName, interTable, joinColumn, alias):
         self.table = table
         self.idName = idName
         self.interTable = interTable
         self.joinColumn = joinColumn
+        self.alias = alias
 
     def tablesUsedImmediate(self):
-        return [self.table, self.interTable]
+        return [
+            '%s  %s' % (self.table, self.alias)
+            if self.alias else self.table,
+            self.interTable,
+        ]
 
     def __sqlrepr__(self, db):
-        return '%s.%s = %s.%s' % (self.interTable, self.joinColumn, self.table,
-                                  self.idName)
+        return '%s.%s = %s.%s' % (
+            self.interTable, self.joinColumn,
+            self.alias if self.alias else self.table, self.idName)
 
 
 class TableToId(sqlbuilder.SQLExpression):
-    def __init__(self, table, idName, idValue):
+    def __init__(self, table, idName, idValue, alias):
         self.table = table
         self.idName = idName
         self.idValue = idValue
+        self.alias = alias
 
     def tablesUsedImmediate(self):
-        return [self.table]
+        return [
+            '%s  %s' % (self.table, self.alias)
+            if self.alias else self.table,
+        ]
 
     def __sqlrepr__(self, db):
-        return '%s.%s = %s' % (self.table, self.idName, self.idValue)
+        return '%s.%s = %s' % (
+            self.alias if self.alias else self.table,
+            self.idName, self.idValue)
+
+
+class SQLJoinSelectResults(sresults.SelectResults):
+    def filter(self, filter_clause):
+        clause_tables = filter_clause.tablesUsed(None)
+        if self._SOSQLRelatedJoin_realSourceClass.sqlmeta.table \
+                in clause_tables:
+            tableClass = self._SOSQLRelatedJoin_realSourceClass.__name__
+            raise ValueError(
+                "Using table '%s' in the filter expression without an alias "
+                "could produce wrong SQL. Most probably you need "
+                "Alias(%s, '_SO_SQLRelatedJoin_OtherTable') instead."
+                % (tableClass, tableClass)
+            )
+        return sresults.SelectResults.filter(self, filter_clause)
 
 
 class SOSQLRelatedJoin(SORelatedJoin):
@@ -339,22 +373,47 @@ class SOSQLRelatedJoin(SORelatedJoin):
             conn = inst._connection
         else:
             conn = None
-        results = self.otherClass.select(sqlbuilder.AND(
-            OtherTableToJoin(
-                self.otherClass.sqlmeta.table, self.otherClass.sqlmeta.idName,
-                self.intermediateTable, self.otherColumn
+        needAlias = self.soClass is self.otherClass
+        if needAlias:
+            source = sqlbuilder.Alias(
+                self.otherClass, '_SO_SQLRelatedJoin_OtherTable')
+            sresultsClass = SQLJoinSelectResults
+        else:
+            source = self.otherClass
+            sresultsClass = self.otherClass.SelectResultsClass
+        results = sresultsClass(
+            source,
+            sqlbuilder.AND(
+                OtherTableToJoin(
+                    self.otherClass.sqlmeta.table,
+                    self.otherClass.sqlmeta.idName,
+                    self.intermediateTable, self.otherColumn,
+                    '_SO_SQLRelatedJoin_OtherTable' if needAlias else '',
+                ),
+                JoinToTable(
+                    self.soClass.sqlmeta.table, self.soClass.sqlmeta.idName,
+                    self.intermediateTable, self.joinColumn,
+                    '_SO_SQLRelatedJoin_ThisTable' if needAlias else '',
+                ),
+                TableToId(
+                    self.soClass.sqlmeta.table, self.soClass.sqlmeta.idName,
+                    inst.id,
+                    '_SO_SQLRelatedJoin_ThisTable' if needAlias else '',
+                ),
             ),
-            JoinToTable(
-                self.soClass.sqlmeta.table, self.soClass.sqlmeta.idName,
-                self.intermediateTable, self.joinColumn
+            clauseTables=(
+                '%s  _SO_SQLRelatedJoin_ThisTable' % self.soClass.sqlmeta.table
+                if needAlias else self.soClass.sqlmeta.table,
+                '%s  _SO_SQLRelatedJoin_OtherTable' %
+                self.otherClass.sqlmeta.table
+                if needAlias else self.otherClass.sqlmeta.table,
+                self.intermediateTable,
             ),
-            TableToId(self.soClass.sqlmeta.table, self.soClass.sqlmeta.idName,
-                      inst.id),
-        ), clauseTables=(self.soClass.sqlmeta.table,
-                         self.otherClass.sqlmeta.table,
-                         self.intermediateTable),
-            connection=conn)
-        return results.orderBy(self.orderBy)
+            connection=conn,
+            orderBy=self.orderBy,
+        )
+        results._SOSQLRelatedJoin_realSourceClass = self.otherClass
+        return results
 
 
 class SQLRelatedJoin(RelatedJoin):
