@@ -21,7 +21,7 @@ class ErrorMessage(str):
             # pg8000 for Python 3.5+
             ecode = eargs0['C']
             eerror = emessage = eargs0['M']
-        elif e.__module__ == 'pg':  # PyGreSQL
+        elif e.__module__ in ('psycopg.errors', 'pg'):  # psycopg, PyGreSQL
             ecode = e.sqlstate
             eerror = emessage = e.args[0]
         elif hasattr(e, 'pgcode'):  # psycopg2 or psycopg2.errors
@@ -57,7 +57,7 @@ class PostgresConnection(DBAPI):
 
     def __init__(self, dsn=None, host=None, port=None, db=None,
                  user=None, password=None, **kw):
-        drivers = kw.pop('driver', None) or 'psycopg'
+        drivers = kw.pop('driver', None) or 'psycopg,psycopg2,pygresql,pg8000'
         for driver in drivers.split(','):
             driver = driver.strip()
             if not driver:
@@ -72,12 +72,13 @@ class PostgresConnection(DBAPI):
                 elif driver == 'pygresql':
                     import pgdb
                     self.module = pgdb
-                elif driver in ('py-postgresql', 'pypostgresql'):
-                    from postgresql.driver import dbapi20
-                    self.module = dbapi20
                 elif driver == 'pg8000':
-                    import pg8000
-                    self.module = pg8000
+                    try:
+                        import pg8000.dbapi
+                        self.module = pg8000.dbapi
+                    except ImportError:
+                        import pg8000
+                        self.module = pg8000
                 elif driver == 'pyodbc':
                     import pyodbc
                     self.module = pyodbc
@@ -94,7 +95,7 @@ class PostgresConnection(DBAPI):
                     raise ValueError(
                         'Unknown PostgreSQL driver "%s", '
                         'expected psycopg, psycopg2, '
-                        'pygresql, pypostgresql, pg8000, '
+                        'pygresql, pg8000, '
                         'odbc, pyodbc or pypyodbc' % driver)
             except ImportError:
                 pass
@@ -104,11 +105,15 @@ class PostgresConnection(DBAPI):
             raise ImportError(
                 'Cannot find a PostgreSQL driver, tried %s' % drivers)
 
-        if driver.startswith('psycopg'):
+        if driver.startswith('psycopg2'):
+            # Register a converter for psycopg2 Binary type.
+            registerConverter(type(self.module.Binary('')),
+                              Psyco2BinaryConverter)
+        elif driver.startswith('psycopg'):
             # Register a converter for psycopg Binary type.
             registerConverter(type(self.module.Binary('')),
                               PsycoBinaryConverter)
-        elif driver in ('pygresql', 'py-postgresql', 'pypostgresql', 'pg8000'):
+        elif driver in ('pygresql', 'pg8000'):
             registerConverter(type(self.module.Binary(b'')),
                               PostgresBinaryConverter)
         elif driver in ('odbc', 'pyodbc', 'pypyodbc'):
@@ -180,13 +185,6 @@ class PostgresConnection(DBAPI):
                     if sslmode:
                         dsn.append('sslmode=%s' % sslmode)
                     dsn = ' '.join(dsn)
-            if driver in ('py-postgresql', 'pypostgresql'):
-                if host and host.startswith('/'):
-                    dsn_dict["host"] = dsn_dict["port"] = None
-                    dsn_dict["unix"] = host
-                else:
-                    if "unix" in dsn_dict:
-                        del dsn_dict["unix"]
             if driver == 'pg8000':
                 if host and host.startswith('/'):
                     dsn_dict["host"] = None
@@ -308,11 +306,6 @@ class PostgresConnection(DBAPI):
         table = soInstance.sqlmeta.table
         idName = soInstance.sqlmeta.idName
         c = conn.cursor()
-        if id is None and self.driver in ('py-postgresql', 'pypostgresql'):
-            sequenceName = soInstance.sqlmeta.idSequence or \
-                '%s_%s_seq' % (table, idName)
-            self._executeRetry(conn, c, "SELECT NEXTVAL('%s')" % sequenceName)
-            id = c.fetchone()[0]
         if id is not None:
             names = [idName] + names
             values = [id] + values
@@ -588,8 +581,8 @@ class PostgresConnection(DBAPI):
         self._createOrDropDatabase(op="DROP")
 
 
-# Converter for Binary types
-def PsycoBinaryConverter(value, db):
+# Converters for Binary types
+def Psyco2BinaryConverter(value, db):
     assert db == 'postgres'
     return str(value)
 
@@ -608,6 +601,11 @@ else:
         )
 
 
+def PsycoBinaryConverter(value, db):
+    assert db == 'postgres'
+    return sqlrepr(escape_bytea(value.obj), db)
+
+
 def PostgresBinaryConverter(value, db):
     assert db == 'postgres'
     return sqlrepr(escape_bytea(value), db)
